@@ -35,15 +35,17 @@ int Board::CreateComponent(int iRow, int iCol, const COMP& eType, const Componen
 	assert( eType != COMP::INVALID );
 
 	// Try and produce a simple unique Name for the new part if possible
-	char buffer[256] = {'\0'};
 	std::string nameStr;	// We'll use this string for both Name and Value
 	const std::string prefixStr = ( pComp ) ? pComp->GetPrefixStr() : CompTypes::GetDefaultPrefixStr(eType);	// e.g. "C" for capacitors
-	bool bNameExists(true);
-	for (int iSuffix = 1; iSuffix < INT_MAX && bNameExists; iSuffix++)
+	if ( !prefixStr.empty() )
 	{
-		sprintf(buffer,"%s%d", prefixStr.c_str(), iSuffix);
-		nameStr = buffer;	// e.g. "C1"
-		bNameExists = ( m_compMgr.GetComponentIdFromName(nameStr) != BAD_COMPID );
+		// Append number to prefixStr
+		bool bNameExists(true);
+		for (int iSuffix = 1; iSuffix < INT_MAX && bNameExists; iSuffix++)
+		{
+			nameStr		= prefixStr + std::to_string(iSuffix);	// e.g. "C1"
+			bNameExists	= ( m_compMgr.GetComponentIdFromName(nameStr) != BAD_COMPID );
+		}
 	}
 
 	const size_t numPins = ( pComp ) ? pComp->GetNumPins() : static_cast<size_t>( CompTypes::GetDefaultNumPins(eType) );
@@ -96,7 +98,8 @@ int Board::AddComponent(int iRow, int iCol, const Component& tmp, bool bDoPlace)
 
 		iRow = std::max(0, std::min(GetRows()-1, iRow));
 		iCol = std::max(0, std::min(GetCols()-1, iCol));
-		const int incRows(iRow + comp.GetRows() - GetRows()), incCols(iCol + comp.GetCols() - GetCols());
+		const int incRows = std::max(iRow + comp.GetRows() - GetRows(), 0);
+		const int incCols = std::max(iCol + comp.GetCols() - GetCols(), 0);
 		if ( incRows > 0 || incCols > 0 )
 			GrowThenPan(0, incRows, incCols, 0, 0);
 
@@ -144,7 +147,8 @@ void Board::AddTextBox(int iRow, int iCol)
 		rect.Move(iRow - rect.m_rowMin, iCol - rect.m_colMin); // Move it to the top left of the view
 	}
 
-	const int incRows(iRow + rect.GetRows() - GetRows()), incCols(iCol + rect.GetCols() - GetCols());
+	const int incRows = std::max(iRow + rect.GetRows() - GetRows(), 0);
+	const int incCols = std::max(iCol + rect.GetCols() - GetCols(), 0);
 	if ( incRows > 0 || incCols > 0 )
 		GrowThenPan(0, incRows, incCols, 0, 0);
 }
@@ -618,7 +622,7 @@ bool Board::TakeOff(Component& comp)
 			}
 		}
 	}
-	if ( bWire )	// Handle wires ends
+	if ( pA != nullptr && pB != nullptr )	// Handle wires ends
 	{
 		const bool bAllLyrs(true);
 
@@ -729,13 +733,11 @@ void Board::PlaceFloaters()	// Try to place down all the floating components
 
 void Board::SelectAllComps(bool bRestrictToRects)
 {
-	const RectManager& rectMgr = GetRectMgr();
-
 	if ( bRestrictToRects )
 	{
 		// Build the trax object
-		Rect bounding = rectMgr.GetBounding() | rectMgr.GetCurrent();
-		m_compMgr.BuildTrax(rectMgr, *this, GetCurrentLayer(), bounding.m_rowMin, bounding.m_rowMax, bounding.m_colMin, bounding.m_colMax);
+		Rect bounding = m_rectMgr.GetBounding() | m_rectMgr.GetCurrent();
+		m_compMgr.BuildTrax(m_rectMgr, *this, GetCurrentLayer(), bounding.m_rowMin, bounding.m_rowMax, bounding.m_colMin, bounding.m_colMax);
 		assert( m_compMgr.GetTrax().GetIsPlaced() );
 	}
 	for (const auto& mapObj : m_compMgr.GetMapIdToComp())
@@ -749,8 +751,7 @@ void Board::SelectAllComps(bool bRestrictToRects)
 			const int	R = comp.GetLastCol();
 			const int	B = comp.GetLastRow();
 
-			const bool bOK = rectMgr.ContainsPoint(T, L) || rectMgr.ContainsPoint(T, R) ||
-							 rectMgr.ContainsPoint(B, L) || rectMgr.ContainsPoint(B, R);
+			const bool bOK = m_rectMgr.Overlaps( Rect(T, B, L, R) );
 			if ( !bOK ) continue;
 		}
 		SetCurrentCompId( mapObj.first );
@@ -773,7 +774,7 @@ bool Board::ConfirmDestroyUserComps()	// returns false if user-group is empty or
 			case COMP::VERO_LETTER:
 			case COMP::WIRE:
 			case COMP::MARK:	continue;
-			default:	assert(eType == COMP::INVALID);	return true;
+			default:	assert(eType != COMP::INVALID);	return true;
 		}
 	}
 	return false;
@@ -793,12 +794,14 @@ void Board::DestroyUserComps()	// Destroy components in the user-group
 	assert( m_groupMgr.GetNumUserComps() == 0 );	// User group should be empty now
 	SetCurrentCompId(BAD_COMPID);
 	PlaceFloaters();		// See if we can now place floating components down
-	GetRectMgr().Clear();
+	m_rectMgr.Clear();
 	m_compMgr.ClearTrax();	// Clear the trax object
 }
 
 void Board::MoveUserCompText(const int& deltaRow, const int& deltaCol)	// Move text label
 {
+	if ( m_groupMgr.GetNumUserComps() != 1 ) return;
+
 	Component& comp	= GetUserComponent();
 	if ( deltaRow == 0 && deltaCol == 0 )	// (0,0) ==> reset rather than shift
 		comp.SetDefaultLabelOffsets();
@@ -808,6 +811,8 @@ void Board::MoveUserCompText(const int& deltaRow, const int& deltaCol)	// Move t
 
 void Board::StretchUserComp(const bool& bGrow)	// Stretch the selected component length
 {
+	if ( m_groupMgr.GetNumUserComps() != 1 ) return;
+
 	Component& comp	= GetUserComponent();
 	if ( bGrow )
 	{
@@ -835,6 +840,8 @@ void Board::StretchUserComp(const bool& bGrow)	// Stretch the selected component
 
 void Board::StretchWidthUserComp(const bool& bGrow)	// Stretch the selected component width (just for DIPs)
 {
+	if ( m_groupMgr.GetNumUserComps() != 1 ) return;
+
 	Component& comp	= GetUserComponent();
 	if ( bGrow )
 	{
@@ -862,6 +869,8 @@ void Board::StretchWidthUserComp(const bool& bGrow)	// Stretch the selected comp
 
 void Board::ChangeTypeUserComp(const COMP& eType)
 {
+	if ( m_groupMgr.GetNumUserComps() != 1 ) return;
+
 	Component& comp	= GetUserComponent();
 	assert( comp.GetType() != eType );	// Current type must differ from new type
 
@@ -952,9 +961,9 @@ bool Board::MoveTextBox(const int& deltaRow, const int& deltaCol)	// Move text b
 	if ( deltaRow == 0 && deltaCol == 0 ) return false;
 
 	// Move the rect and update the manager
-	GetTextMgr().MoveRect(GetCurrentTextId(), deltaRow, deltaCol);
+	m_textMgr.MoveRect(GetCurrentTextId(), deltaRow, deltaCol);
 
-	TextRect& rect = GetTextMgr().GetTextRectById( GetCurrentTextId() );
+	TextRect& rect = m_textMgr.GetTextRectById( GetCurrentTextId() );
 	if ( !rect.GetIsValid() ) return false;
 
 	// Pan the circuit as needed if the rect has gone out of bounds
@@ -1194,7 +1203,7 @@ void Board::FixCorruption()
 	PlaceFloaters();	// Unfloat components
 }
 
-Rect Board::GetFootprintBounds(const std::list<int>& compIds)
+Rect Board::GetFootprintBounds(const std::list<int>& compIds) const
 {
 	Rect bounding;
 	for (const auto& compId : compIds)
