@@ -58,7 +58,7 @@ public:
 	{
 		m_mapIdToComp.clear();
 		m_mapWireToInfo.clear();
-		m_foundId.clear();
+		ClearFind();
 		ClearTrax();
 	}
 	CompManager& operator=(const CompManager& o)
@@ -96,17 +96,13 @@ public:
 		}
 		return BAD_COMPID;
 	}
+	const Component& GetComponentById(const int& compId) const
+	{
+		return LookupCompById(compId);
+	}
 	Component& GetComponentById(const int& compId)
 	{
-		if ( compId == TRAX_COMPID ) return m_trax;
-
-		auto iter = m_mapIdToComp.find(compId);
-		if ( iter != m_mapIdToComp.end() ) return iter->second;
-
-		// Should not really get here!!!  Use assert() to check the returned component type is valid.
-		Component& comp = m_mapIdToComp[compId];	// This creates a blank component and puts it in the map
-		comp.SetId(compId);	// Even a blank component should have the correct compId
-		return comp;
+		return const_cast<Component&> ( LookupCompById(compId) );
 	}
 	bool GetAllowFlyWire(const int& compId) const
 	{
@@ -324,10 +320,14 @@ public:
 			if ( CompTypes::AllowCustomPCBshapes(comp.GetType()) ) comp.SetDefaultShapes(bUsePCBshapes);
 		}
 	}
-	void Find(const bool bUseName, const bool bExact, const std::string& str)
+	void ClearFind()
 	{
 		m_foundId.clear();
-		if ( str.empty() ) return;
+	}
+	void Find(const bool bUseName, const bool bExact, const std::string& str)
+	{
+		ClearFind();
+		if ( str.empty() ) return;	// Don't search with an empty string
 		for (const auto& mapObj : m_mapIdToComp)
 		{
 			const Component&	comp	= mapObj.second;
@@ -343,6 +343,10 @@ public:
 	{
 		return m_foundId.find(compId) != m_foundId.end();
 	}
+	size_t GetNumFound() const
+	{
+		return m_foundId.size();
+	}
 
 	// Merge interface functions
 	virtual void UpdateMergeOffsets(MergeOffsets& o) override
@@ -369,13 +373,11 @@ public:
 			{
 				// Try and produce a simple unique Name for the part if possible
 				std::string nameStr;
-				char buffer[256] = {'\0'};
 				const std::string prefixStr = comp.GetPrefixStr();	// e.g. "C" for capacitors
 				for (int iSuffix = 1; iSuffix < INT_MAX && bNameExists; iSuffix++)
 				{
-					sprintf(buffer,"%s%d", prefixStr.c_str(), iSuffix);
-					nameStr = buffer;	// e.g. "C1"
-					bNameExists = GetComponentIdFromName(nameStr) != BAD_COMPID;
+					nameStr		= prefixStr + std::to_string(iSuffix);	// e.g. "C1"
+					bNameExists	= GetComponentIdFromName(nameStr) != BAD_COMPID;
 				}
 				comp.SetNameStr(nameStr);
 			}
@@ -409,9 +411,10 @@ public:
 		for (auto& mapObj : m_mapIdToComp) mapObj.second.Save(outStream);
 		m_trax.Save(outStream);	// Added in VRT_VERSION_11
 	}
-	Component&	GetTrax()	{ return m_trax; }
-	void		ClearTrax()	{ m_trax.DeAllocate(); m_trax.SetType(COMP::TRACKS); m_trax.SetId(TRAX_COMPID); m_trax.SetIsPlaced(false); m_trax.SetRow(0); m_trax.SetCol(0); }
-	void		BuildTrax(const RectManager& rectMgr, const ElementGrid& grid, const int& nLyr, const int& nRowMin, const int& nRowMax, const int& nColMin, const int& nColMax)
+	const Component&	GetTrax() const	{ return m_trax; }
+	Component&			GetTrax()		{ return m_trax; }
+	void				ClearTrax()	{ m_trax.DeAllocate(); m_trax.SetType(COMP::TRACKS); m_trax.SetId(TRAX_COMPID); m_trax.SetIsPlaced(false); m_trax.SetRow(0); m_trax.SetCol(0); }
+	void				BuildTrax(const RectManager& rectMgr, const ElementGrid& grid, const int& nLyr, const int& nRowMin, const int& nRowMax, const int& nColMin, const int& nColMax)
 	{
 		m_trax = Component(this, rectMgr, grid, nLyr, nRowMin, nRowMax, nColMin, nColMax);
 		m_trax.SetId(TRAX_COMPID);
@@ -422,6 +425,18 @@ private:
 	{
 		return m_mapIdToComp.find(compId) != m_mapIdToComp.end();
 	}
+	const Component& LookupCompById(const int& compId) const
+	{
+		if ( compId == TRAX_COMPID ) return m_trax;
+
+		auto iter = m_mapIdToComp.find(compId);
+		if ( iter != m_mapIdToComp.end() ) return iter->second;
+
+		// Should not really get here!!!  Use assert() to check the returned component type is valid.
+		Component& comp = m_mapIdToComp[compId];	// This creates a blank component and puts it in the map
+		comp.SetId(compId);	// Even a blank component should have the correct compId
+		return comp;
+	}
 	struct IsEarlierWire
 	{
 		bool operator()(const Component* pA, const Component* pB) const
@@ -450,7 +465,7 @@ private:
 		}
 	};
 private:
-	std::unordered_map<int, Component>				m_mapIdToComp;		// The components (indexed by compId)
+	mutable std::unordered_map<int, Component>		m_mapIdToComp;		// The components (indexed by compId).  Mutable because of LookupCompById()
 	Component										m_trax;				// The "trax" component
 	// Helpers. Don't persist.
 	std::unordered_map<const Component*, WireInfo>	m_mapWireToInfo;	// For handling overlaid / crossing wires
