@@ -190,7 +190,7 @@ struct CompTypes
 		UpdateMaps(COMP::FUSE_HOLDER,			"Fuse Holder",					"FUSE_HOLDER");
 		UpdateMaps(COMP::VERO_NUMBER,			"Vero Numbers",					"");	// No import string
 		UpdateMaps(COMP::VERO_LETTER,			"Vero Letters",					"");	// No import string
-		UpdateMaps(COMP::CUSTOM,				"Custom",						"");	// No import string (user-defined shapes have their own strings)
+		UpdateMaps(COMP::CUSTOM,				"Custom",						"");	// No import string (user-defined parts have their own strings)
 		UpdateMaps(COMP::TRACKS,				"Tracks",						"");	// No import string
 	}
 	static const std::list<COMP>& GetListCompTypes()
@@ -520,9 +520,9 @@ struct CompTypes
 			case COMP::PAD					: rows = 1; cols = 1;  return "1";
 			case COMP::PAD_FLYINGWIRE		: rows = 1; cols = 1;  return "1";
 			case COMP::WIRE					: rows = 1; cols = 3;  return "1+2";
-			case COMP::RESISTOR				: rows = 1; cols = 5;  return "1+++2";
-			case COMP::INDUCTOR				: rows = 1; cols = 5;  return "1+++2";
-			case COMP::DIODE				: rows = 1; cols = 5;  return "1+++2";
+			case COMP::RESISTOR				: rows = 1; cols = 4;  return "1++2";
+			case COMP::INDUCTOR				: rows = 1; cols = 4;  return "1++2";
+			case COMP::DIODE				: rows = 1; cols = 4;  return "1++2";
 			case COMP::LED					: rows = 1; cols = 2;  return "12";
 			case COMP::CAP_CERAMIC			:
 			case COMP::CAP_FILM				: rows = 1; cols = 3;  return "1+2";
@@ -597,9 +597,7 @@ struct CompTypes
 	}
 	static std::string GetDefaultPinLabel(size_t iPinIndex)
 	{
-		static char buffer[32];
-		sprintf(buffer, "%d", static_cast<int>(iPinIndex+1));	// Pin numbers on screen start at 1
-		return buffer;
+		return std::to_string(iPinIndex + 1);	// Pin numbers on screen start at 1
 	}
 	static int GetDefaultPinAlign(size_t iPinIndex, size_t iNumPins, const COMP& eType)
 	{
