@@ -41,7 +41,7 @@ class CompManager;
 class Component : public FootPrint
 {
 public:
-	Component()	: FootPrint() { Clear(); }
+	Component() : FootPrint() { Clear(); }
 	virtual ~Component() override {}
 	Component(const Component& o) : FootPrint() { *this = o; }
 	void Clear()
@@ -185,7 +185,7 @@ public:
 				&& m_bAllowFlyWire		== o.m_bAllowFlyWire
 				&& GetNumPins()			== o.GetNumPins()
 				&& GetNumShapes()		== o.GetNumShapes();
-		for (size_t i = 0; i < GetNumPins() && bOK; i++)
+		for (size_t i = 0, iSize = GetNumPins(); i < iSize && bOK; i++)
 		{
 			bOK =  m_nodeIdPins[i]		== o.m_nodeIdPins[i]
 				&& m_origIdPins1[i]		== o.m_origIdPins1[i]
@@ -196,7 +196,7 @@ public:
 				&& m_pinLabels[i]		== o.m_pinLabels[i]
 				&& m_pinAligns[i]		== o.m_pinAligns[i];
 		}
-		for (size_t i = 0; i < GetNumShapes() && bOK; i++)
+		for (size_t i = 0, iSize = GetNumShapes(); i < iSize && bOK; i++)
 		{
 			bOK = m_shapes[i] == o.m_shapes[i];
 		}
@@ -400,11 +400,9 @@ public:
 	{
 		std::string str = GetTypeStr();
 		if ( GetType() == COMP::DIP || GetType() == COMP::SIP )
-		{
-			char buffer[32] = {'\0'};
-			sprintf(buffer, "%d", static_cast<int>(GetNumPins()));
-			str += std::string(buffer);		// e.g. "DIP16"
-		}
+			str += std::to_string(GetNumPins());	// e.g. "DIP16"
+		if ( GetType() == COMP::STRIP_100 || GetType() == COMP::BLOCK_100 || GetType() == COMP::BLOCK_200 )
+			str += std::string(" (") + std::to_string(GetNumPins()) + std::string(" pins)");
 		return str;
 	}
 	std::string GetFullImportStr() const	// For SIP/DIP/SWITCH/STRIP/BLOCK types, append the number of pins
@@ -413,11 +411,7 @@ public:
 		if ( GetType() == COMP::DIP || GetType() == COMP::SIP ||
 			 GetType() == COMP::SWITCH_DT || GetType() == COMP::SWITCH_ST || GetType() == COMP::SWITCH_ST_DIP ||
 			 GetType() == COMP::STRIP_100 || GetType() == COMP::BLOCK_100 || GetType() == COMP::BLOCK_200 )
-		{
-			char buffer[32] = {'\0'};
-			sprintf(buffer, "%d", static_cast<int>(GetNumPins()));
-			str += std::string(buffer);		// e.g. "DIP16"
-		}
+			str += std::to_string(GetNumPins());	// e.g. "DIP16"
 		return str;
 	}
 	// Helpers (account for component direction)
@@ -447,6 +441,8 @@ public:
 	}
 	void IncCompPinOffsets(const size_t& iPinIndex, const int& dX, const int& dY)
 	{
+		if ( dX == 0 && dY == 0 )
+			return SetCompPinOffsets(iPinIndex, 0, 0);	// Reset
 		int Xmil, Ymil;
 		GetCompPinOffsets(iPinIndex, Xmil, Ymil);
 		SetCompPinOffsets(iPinIndex, Xmil + dX, Ymil + dY);
@@ -553,7 +549,7 @@ public:
 	}
 	void SetDefaultPinLabels()
 	{
-		for (size_t i = 0; i < GetNumPins(); i++)
+		for (size_t i = 0, iSize = GetNumPins(); i < iSize; i++)
 		{
 			m_pinLabels[i] = CompTypes::GetDefaultPinLabel(i);
 			m_pinAligns[i] = CompTypes::GetDefaultPinAlign(i, GetNumPins(), GetType());
