@@ -18,7 +18,6 @@
 */
 
 #include "CurveList.h"
-#include <QPolygon>
 
 Curve::Curve(const QPoint& p, const GPEN& ePen, const int& width) : m_ePen(ePen), m_width(width)
 {
@@ -79,6 +78,12 @@ bool Curve::Splice(Curve* pB)	// Tries to splice curve B to this
 	return false;
 }
 
+void CurveList::Clear()
+{
+	for (auto& p : *this) p->clear();
+	clear();
+}
+
 void CurveList::Sort()
 {
 	sort( Curve::HasLargerPen() );
