@@ -141,6 +141,7 @@ public:
 	void SetSolderR(const bool& b)		{ auto pBase = GetBase(); if ( pBase == this ) m_bSolderR = b; else pBase->SetSolderR(b); }
 	void SetIsVia(const bool& b)		{ auto pBase = GetBase(); if ( pBase == this ) m_bIsVia	  = b; else pBase->SetIsVia(b); }
 	void SetRoutable(const int& i)		{ m_iRoutable	= i; }
+	void SetRouteId(const unsigned int& i)	{ m_routeId = i; }
 	void ResetMH()
 	{
 		m_routeId	= BAD_ROUTEID;	// Wipe RouteId
@@ -215,6 +216,7 @@ public:
 		assert( iSlot == 0 || iSlot == 1 );
 		auto pBase = GetBase();	if ( pBase == this ) m_pW[iSlot] = p; else pBase->SetW(iSlot, p);
 	}
+	void				SetMH(const unsigned int& iMH)	{ m_MH = iMH; }
 	const bool&			GetIsMark() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_bIsMark		: pBase->GetIsMark(); }
 	const int&			GetCompId() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_compId		: pBase->GetCompId(); }
 	const int&			GetCompId2() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_compId2		: pBase->GetCompId2(); }
@@ -269,17 +271,21 @@ public:
 		pW = pNbr->GetW(0);		if ( pW ) { pW->WipeFlagBits(AUTOSET|VEROSET);	pW->MarkFlagBits(USERSET); }
 		pW = pNbr->GetW(1);		if ( pW ) { pW->WipeFlagBits(AUTOSET|VEROSET);	pW->MarkFlagBits(USERSET); }
 	}
-	bool SwapDiagLinks()
+	bool CanSwapDiagLinks()
 	{
 		// Take "this" to be the bottom right element in group of 4 squares
 		// "LT"  is the diagonal from "this" to m_pLT
 		// "LTX" is the diagonal that cuts across it (from m_pL to m_pT)
-
-		// Swap (by inverting flags) if we have competing diagonals
 		const bool bCanSwap = GetNodeId() == GetNbr(NBR_LT)->GetNodeId()				&&	// LT:  "this" and LT must have same nodeId
 							  GetNbr(NBR_L)->GetNodeId() == GetNbr(NBR_T)->GetNodeId()	&&	// LTX: L and T must have same nodeId
 							  GetNbr(NBR_L)->IsClash( GetNodeId() );						// L and "this" must have clashing nodeIds
-		if ( !bCanSwap ) return false;
+		return bCanSwap;
+	}
+	bool SwapDiagLinks()
+	{
+		if ( !CanSwapDiagLinks() ) return false;
+
+		// Swap (by inverting flags) if we have competing diagonals
 		ToggleUsed(NBR_LT);
 		GetNbr(NBR_L)->ToggleUsed(NBR_RT);
 		return true;
