@@ -57,6 +57,7 @@ public:
 		m_GRIDPIXELS		= o.m_GRIDPIXELS;
 		m_PAD_MIL			= o.m_PAD_MIL;
 		m_TRACK_MIL			= o.m_TRACK_MIL;
+		m_TAG_MIL			= o.m_TAG_MIL;
 		m_HOLE_MIL			= o.m_HOLE_MIL;
 		m_GAP_MIL			= o.m_GAP_MIL;
 		m_MASK_MIL			= o.m_MASK_MIL;
@@ -91,6 +92,7 @@ public:
 		m_bGroundFill		= o.m_bGroundFill;
 		m_bVerticalStrips	= o.m_bVerticalStrips;
 		m_bCompEdit			= o.m_bCompEdit;
+		m_bXthermals		= o.m_bXthermals;
 		return *this;
 	}
 	bool operator==(const GuiControl& o) const	// Compare persisted info
@@ -109,6 +111,7 @@ public:
 			&&	m_GRIDPIXELS		== o.m_GRIDPIXELS
 			&&	m_PAD_MIL			== o.m_PAD_MIL
 			&&	m_TRACK_MIL			== o.m_TRACK_MIL
+			&&	m_TAG_MIL			== o.m_TAG_MIL
 			&&	m_HOLE_MIL			== o.m_HOLE_MIL
 			&&	m_GAP_MIL			== o.m_GAP_MIL
 			&&	m_MASK_MIL			== o.m_MASK_MIL
@@ -142,7 +145,8 @@ public:
 			&&	m_bFlyWires			== o.m_bFlyWires
 			&&	m_bGroundFill		== o.m_bGroundFill
 			&&	m_bVerticalStrips	== o.m_bVerticalStrips
-			&&	m_bCompEdit			== o.m_bCompEdit;
+			&&	m_bCompEdit			== o.m_bCompEdit
+			&&	m_bXthermals		== o.m_bXthermals;
 	}
 	bool operator!=(const GuiControl& o) const
 	{
@@ -220,6 +224,9 @@ public:
 		inStream.Load(m_GRIDPIXELS);
 		inStream.Load(m_PAD_MIL);
 		inStream.Load(m_TRACK_MIL);
+		m_TAG_MIL = m_TRACK_MIL;
+		if ( inStream.GetVersion() >= VRT_VERSION_52 )
+			inStream.Load(m_TAG_MIL);			// Added in VRT_VERSION_52
 		inStream.Load(m_HOLE_MIL);
 		m_GAP_MIL = 10;
 		if ( inStream.GetVersion() >= VRT_VERSION_3 )
@@ -312,6 +319,9 @@ public:
 		m_bCompEdit = false;
 		if ( inStream.GetVersion() >= VRT_VERSION_19 )
 			inStream.Load(m_bCompEdit);			// Added in VRT_VERSION_19
+		m_bXthermals = false;
+		if ( inStream.GetVersion() >= VRT_VERSION_53 )
+			inStream.Load(m_bXthermals);		// Added in VRT_VERSION_53
 	}
 	virtual void Save(DataStream& outStream) override
 	{
@@ -329,6 +339,7 @@ public:
 		outStream.Save(m_GRIDPIXELS);
 		outStream.Save(m_PAD_MIL);
 		outStream.Save(m_TRACK_MIL);
+		outStream.Save(m_TAG_MIL);			// Added in VRT_VERSION_52
 		outStream.Save(m_HOLE_MIL);
 		outStream.Save(m_GAP_MIL);			// Added in VRT_VERSION_3
 		outStream.Save(m_MASK_MIL);			// Added in VRT_VERSION_32
@@ -363,6 +374,7 @@ public:
 		outStream.Save(m_bGroundFill);		// Added in VRT_VERSION_3
 		outStream.Save(m_bVerticalStrips);	// Added in VRT_VERSION_12
 		outStream.Save(m_bCompEdit);		// Added in VRT_VERSION_19
+		outStream.Save(m_bXthermals);		// Added in VRT_VERSION_53
 	}
 	bool SetBackgroundColor(const MyRGB& o)	{ const bool bChanged = m_backgroundColor	!= o; m_backgroundColor	= o; return bChanged;}
 	bool SetCurrentLayer(const int& i)		{ const bool bChanged = m_currentLayer		!= i; m_currentLayer	= i; return bChanged; }
@@ -380,6 +392,7 @@ public:
 											  if ( bChanged && GetHOLE_MIL() > i-8 ) SetHOLE_MIL( i-8 );		// 8 ==> minimum annular ring = 4 mil
 											  return bChanged; }
 	bool SetTRACK_MIL(const int& i)			{ const bool bChanged = m_TRACK_MIL			!= i; m_TRACK_MIL		= i; return bChanged; }
+	bool SetTAG_MIL(const int& i)			{ const bool bChanged = m_TAG_MIL			!= i; m_TAG_MIL			= i; return bChanged; }
 	bool SetHOLE_MIL(const int& i)			{ const bool bChanged = m_HOLE_MIL			!= i; m_HOLE_MIL		= i;
 											  if ( bChanged && GetPAD_MIL() < i+8 ) SetPAD_MIL( i+8 );			// 8 ==> minimum annular ring = 4 mil
 											  return bChanged; }
@@ -421,6 +434,7 @@ public:
 	bool SetGroundFill(const bool& b)		{ const bool bChanged = m_bGroundFill		!= b; m_bGroundFill		= b; return bChanged; }
 	bool SetVerticalStrips(const bool& b)	{ const bool bChanged = m_bVerticalStrips	!= b; m_bVerticalStrips	= b; return bChanged; }
 	bool SetCompEdit(const bool& b)			{ const bool bChanged = m_bCompEdit			!= b; m_bCompEdit		= b; return bChanged; }
+	bool SetXthermals(const bool& b)		{ const bool bChanged = m_bXthermals		!= b; m_bXthermals		= b; return bChanged; }
 	const MyRGB&		GetBackgroundColor() const	{ return m_backgroundColor; }
 	const int&			GetCurrentLayer() const		{ return m_currentLayer; }
 	const int&			GetCurrentNodeId() const	{ return m_currentNodeId; }
@@ -435,6 +449,7 @@ public:
 	const int&			GetGRIDPIXELS() const		{ return m_GRIDPIXELS; }
 	const int&			GetPAD_MIL() const			{ return m_PAD_MIL; }
 	const int&			GetTRACK_MIL() const		{ return m_TRACK_MIL; }
+	const int&			GetTAG_MIL() const			{ return m_TAG_MIL; }
 	const int&			GetHOLE_MIL() const			{ return m_HOLE_MIL; }
 	const int&			GetGAP_MIL() const			{ return m_GAP_MIL; }
 	const int&			GetMASK_MIL() const			{ return m_MASK_MIL; }
@@ -469,16 +484,19 @@ public:
 	const bool&			GetGroundFill() const		{ return m_bGroundFill; }
 	const bool&			GetVerticalStrips() const	{ return m_bVerticalStrips; }
 	const bool&			GetCompEdit() const			{ return m_bCompEdit; }
+	const bool&			GetXthermals() const		{ return m_bXthermals; }
 	// Helpers
-	bool	GetUsePCBshapes()			{ return GetTrackMode() == TRACKMODE::PCB; }
-	bool	GetMirrored() const			{ return GetFlipH() || GetFlipV(); }
-	bool	SetTrackSliderValue(int i)	{ const bool bChanged = ( GetTrackSliderValue() != i ); SetTrackMode( static_cast<TRACKMODE>(i) ); return bChanged; }
-	bool	SetCompSliderValue(int i)	{ const bool bChanged = ( GetCompSliderValue()  != i ); SetCompMode(  static_cast<COMPSMODE>(i) ); return bChanged; }
-	int		GetTrackSliderValue() const	{ return static_cast<int>(GetTrackMode()); }
-	int		GetCompSliderValue() const	{ return static_cast<int>(GetCompMode());  }
-	double	GetSilkWidth() const		{ return std::max(1.0, GetGRIDPIXELS() * GetSILK_MIL() * 0.010 );  }	// Silk-screen pen width in pixels
-	double	GetEdgeWidth() const		{ return std::max(1.0, GetGRIDPIXELS() * GetEDGE_MIL() * 0.010 );  }	// Board edge margin in pixels
-	int		GetHalfPixelsFromMIL(const int& iMIL) const
+	bool		SetGroundNodeId()				{ return ( GetCurrentLayer() == 0 ) ? SetGroundNodeId0( GetCurrentNodeId() ) : SetGroundNodeId1( GetCurrentNodeId() ); }
+	const int&	GetGroundNodeId(int lyr) const	{ return ( lyr == 0 ) ? GetGroundNodeId0() : GetGroundNodeId1(); }
+	bool		GetUsePCBshapes()				{ return GetTrackMode() == TRACKMODE::PCB; }
+	bool		GetMirrored() const				{ return GetFlipH() || GetFlipV(); }
+	bool		SetTrackSliderValue(int i)		{ const bool bChanged = ( GetTrackSliderValue() != i ); SetTrackMode( static_cast<TRACKMODE>(i) ); return bChanged; }
+	bool		SetCompSliderValue(int i)		{ const bool bChanged = ( GetCompSliderValue()  != i ); SetCompMode(  static_cast<COMPSMODE>(i) ); return bChanged; }
+	int			GetTrackSliderValue() const		{ return static_cast<int>(GetTrackMode()); }
+	int			GetCompSliderValue() const		{ return static_cast<int>(GetCompMode());  }
+	double		GetSilkWidth() const			{ return std::max(1.0, GetGRIDPIXELS() * GetSILK_MIL() * 0.010 );  }	// Silk-screen pen width in pixels
+	double		GetEdgeWidth() const			{ return std::max(1.0, GetGRIDPIXELS() * GetEDGE_MIL() * 0.010 );  }	// Board edge margin in pixels
+	int			GetHalfPixelsFromMIL(const int& iMIL) const
 	{
 		return std::max(1, static_cast<int> (GetGRIDPIXELS() * iMIL	* 0.005 ));
 	}
@@ -486,7 +504,39 @@ public:
 	{
 		return std::max(1, static_cast<int> (GetGRIDPIXELS() * iMIL	* 0.010 ));
 	}
-	void	CalcBlob(const qreal& W, const QPointF& pC, const QPointF& pCoffset, const int& iPerimeterCode, std::list<MyPolygonF>& out, const bool bHavePad = false, const bool bGap = false) const;
+	void	CalcBlob(const qreal& W, const QPointF& pC, const QPointF& pCoffset,
+					 const int& iPadWidthMIL, const int& iPerimeterCode, const int& iTagCode,
+					 std::list<MyPolygonF>& out,
+					 const bool bHavePad, const bool bIsGnd, const bool bGap = false) const;
+	void Reset()	// For use with File->New()
+	{
+		m_currentLayer		= 0;
+		m_currentCompId		= BAD_COMPID;
+		m_currentNodeId		= BAD_NODEID;
+		m_groundNodeId0		= BAD_NODEID;
+		m_groundNodeId1		= BAD_NODEID;
+		m_currentTextId		= BAD_TEXTID;
+		m_diagsMode			= DIAGSMODE::MIN;
+		m_iTrackMode		= TRACKMODE::COLOR;
+		m_iCompMode			= COMPSMODE::NAME;
+		m_iFillSaturation	= 0;
+		m_bShowTarget		= false;
+		m_bShowCloseTracks	= false;
+		m_bVeroTracks		= false;
+		m_bCurvedTracks		= false;
+		m_bFatTracks		= true;
+		m_bRoutingEnabled	= false;
+		m_bViasEnabled		= true;
+		m_bShowGrid			= true;
+		m_bShowText			= true;
+		m_bFlipH			= false;
+		m_bFlipV			= false;
+		m_bPinLabels		= true;
+		m_bFlyWires			= true;
+		m_bGroundFill		= false;
+		m_bCompEdit			= false;
+		m_bXthermals		= false;
+	}
 private:
 	MyRGB		m_backgroundColor	= MyRGB(0xFFFFFF);
 	int			m_currentLayer		= 0;				// Currently selected layer for display
@@ -502,6 +552,7 @@ private:
 	int			m_GRIDPIXELS		= 24;				// Default 24 pixels per grid square (i.e. per 100 mil)
 	int			m_PAD_MIL			= 60;				// Range 50 to 98
 	int			m_TRACK_MIL			= 24;				// Range 12 to 50
+	int			m_TAG_MIL			= 12;				// Range 12 to 50 (Same range as m_TRACK_MIL for backward compatibility)
 	int			m_HOLE_MIL			= 34;				// Range 20 to 40
 	int			m_GAP_MIL			= 10;				// Range  5 to 30
 	int			m_MASK_MIL			= 4;				// Range  0 to 10
@@ -536,4 +587,5 @@ private:
 	bool		m_bGroundFill		= false;
 	bool		m_bVerticalStrips	= true;
 	bool		m_bCompEdit			= false;			// true ==> component editor mode
+	bool		m_bXthermals		= false;			// true ==> force X thermal reliefs and hide tracks in ground-fill
 };
