@@ -214,7 +214,7 @@ public:
 	}
 	static const std::map<SHAPE, std::string>& GetMapShapeStrings()
 	{
-		static std::map<SHAPE, std::string>	mapShapeToStr;	// Mapping of SHAPE enum to strings
+		static std::map<SHAPE, std::string>	mapShapeToStr;
 		if ( mapShapeToStr.empty() )
 		{
 			mapShapeToStr[SHAPE::LINE]			= "Line";
@@ -226,6 +226,20 @@ public:
 		}
 		return mapShapeToStr;
 	}
+	static const std::list<std::string>& GetListShapeStrings()
+	{
+		static std::list<std::string>	listShapeStr;	// Use for populating combo boxes
+		if ( listShapeStr.empty() )
+		{
+			listShapeStr.push_back("Line");
+			listShapeStr.push_back("Rectangle");
+			listShapeStr.push_back("Rounded Rectangle");
+			listShapeStr.push_back("Ellipse");
+			listShapeStr.push_back("Arc");
+			listShapeStr.push_back("Chord");
+		}
+		return listShapeStr;
+	}
 private:
 	// Data (x1,y1) and (x2,y2) are either
 	// the start and end points of a line, or the TL and BR corners of a rectangle.
