@@ -19,6 +19,7 @@
 
 #pragma once
 
+#include "VeroRouteAndroid.h"
 #include <QtGui>
 #include <QApplication>
 #include <QColorDialog>
@@ -36,6 +37,10 @@
 #include "GWriter.h"
 #include "myscrollarea.h"
 
+#ifndef VEROROUTE_ANDROID
+#define USE_PIXMAP_CACHE
+#endif
+
 namespace Ui { class MainWindow; }
 
 class ControlDialog;
@@ -48,10 +53,16 @@ class CompDialog;
 class TextDialog;
 class BomDialog;
 class PinDialog;
+class PadOffsetDialog;
 class FindDialog;
 
 const size_t MAX_RECENT_FILES = 10;
 
+#ifdef VEROROUTE_ANDROID
+#define ANDROID_VSCROLL_WIDTH  QString("QScrollBar:vertical { width: 20px; }")
+#define ANDROID_HSCROLL_HEIGHT QString("QScrollBar:horizontal { height: 20px; }")
+#endif
+
 class MainWindow : public QMainWindow
 {
 	friend class ControlDialog;
@@ -64,20 +75,37 @@ class MainWindow : public QMainWindow
 	friend class TextDialog;
 	friend class BomDialog;
 	friend class PinDialog;
+	friend class PadOffsetDialog;
 	friend class FindDialog;
+	friend class MyScrollArea;
 
 	Q_OBJECT
 
 public:
+	enum class MOUSE_MODE
+	{
+		SELECT			= 0,	// Select objects (parts, text boxes, etc)
+		PAINT_PINS		= 1,	// Paint component pins (and the board)
+		ERASE_PINS		= 2,	// Erase component pins (and the board)
+		PAINT_GRID		= 3,	// Paint grid points only (not component pins)
+		ERASE_GRID		= 4,	// Erase grid points only (not component pins)
+		PAINT_FLOOD		= 5,	// Flood-fill all connected tracks & pins
+		DEFINE_RECT		= 6,	// Define grey rectangle areas
+		RESIZE_TEXT		= 7,	// Resize text rectangle
+		SMART_PAN		= 8		// Move whole layout and resize/crop grid
+	};
+
 	explicit MainWindow(const QString& localDataPathStr, const QString& tutorialsPathStr, QWidget* parent = nullptr);
 	~MainWindow();
 
-	void ResetView(bool bTutorial = false);
+	void ResetView(MOUSE_MODE eMouseMode = MOUSE_MODE::SELECT, bool bTutorial = false);
 	void CheckFolders();
+	void CheckHistory();
 	// Helper for mouse
 	void GetPixMapXY(const QPoint& currentPoint, int& pixmapX, int& pixmapY) const;
-	void GetRowCol(const QPoint& currentPoint, int& row, int& col, double& deltaRow, double& deltaCol) const;
-	void GetRowCol(const QPoint& currentPoint, const int rows, const int cols, int& row, int& col, double& deltaRow, double& deltaCol) const;
+	bool GetRowCol(const QPoint& currentPoint, int& row, int& col, double& deltaRow, double& deltaCol) const;
+	bool GetRowCol(const QPoint& currentPoint, const int rows, const int cols, int& row, int& col, double& deltaRow, double& deltaCol) const;
+	bool HaveZeroDeltaRowCol(int& deltaRow, int& deltaCol);
 
 	// Helpers for rendering
 	void GetFirstRowCol(int& iRow, int& iCol) const;
@@ -90,34 +118,48 @@ public:
 
 	const bool&	GetCtrlKeyDown() const	{ return m_bCtrlKeyDown;	}
 	const bool&	GetShiftKeyDown() const	{ return m_bShiftKeyDown;	}
-	const bool&	GetPaintPins() const	{ return m_bPaintPins;		}
-	const bool&	GetPaintBoard() const	{ return m_bPaintBoard;		}
-	const bool&	GetPaintFlood() const	{ return m_bPaintFlood;		}
-	const bool&	GetPaintLyrPref() const	{ return m_bPaintLyrPref;	}
-	const bool&	GetDefiningRect() const	{ return m_bDefiningRect;	}
-	const bool&	GetResizingText() const	{ return m_bResizingText;	}
+
+	bool		GetPaintAction() const	{ return m_eMouseMode >= MOUSE_MODE::PAINT_PINS && m_eMouseMode <= MOUSE_MODE::PAINT_FLOOD; }
+	bool		GetPaintPins() const	{ return m_eMouseMode == MOUSE_MODE::PAINT_PINS; }
+	bool		GetErasePins() const	{ return m_eMouseMode == MOUSE_MODE::ERASE_PINS; }
+	bool		GetPaintBoard() const	{ return m_eMouseMode == MOUSE_MODE::PAINT_GRID; }
+	bool		GetEraseBoard() const	{ return m_eMouseMode == MOUSE_MODE::ERASE_GRID; }
+	bool		GetPaintFlood() const	{ return m_eMouseMode == MOUSE_MODE::PAINT_FLOOD; }
+	bool		GetDefiningRect() const	{ return m_eMouseMode == MOUSE_MODE::DEFINE_RECT; }
+	bool		GetResizingText() const	{ return m_eMouseMode == MOUSE_MODE::RESIZE_TEXT; }
+	bool		GetSmartPan() const		{ return m_eMouseMode == MOUSE_MODE::SMART_PAN || GetCtrlKeyDown(); }
+
 	void		SetCtrlKeyDown(bool b)	{ m_bCtrlKeyDown	= b; }
 	void		SetShiftKeyDown(bool b)	{ m_bShiftKeyDown	= b; }
 	void		SetPaintPins(bool b);
+	void		SetErasePins(bool b);
 	void		SetPaintBoard(bool b);
+	void		SetEraseBoard(bool b);
 	void		SetPaintFlood(bool b);
-	void		SetPaintLyrPref(bool b);
 	void		SetDefiningRect(bool b);
-	void		SetResizingText(bool b)	{ m_bResizingText	= b; }
+	void		SetResizingText(bool b);
+	void		SetSmartPan(bool b);
+
+	const QString&	GetFileName() const { return m_fileName; }
+
+	bool GetHaveFloatingPin(int& iFloatingNodeId);
+	void MousePressEvent(const QPoint& pos, const bool& bLeftClick = true, const bool& bRightClick = true);
+	void MouseDoubleClickEvent(const QPoint& pos);
+	void MouseMoveEvent(const QPoint& pos);
+	void MouseReleaseEvent(const QPoint& pos);
 protected:
 	void paintEvent(QPaintEvent* event);
 	void wheelEvent(QWheelEvent* event);
-	void mousePressEvent(QMouseEvent* event);
-	void mouseDoubleClickEvent(QMouseEvent* event);
-	void mouseMoveEvent(QMouseEvent* event);
-	void mouseReleaseEvent(QMouseEvent* event);
 	void keyPressEvent(QKeyEvent* event);
 	void keyReleaseEvent(QKeyEvent* event);
+#ifndef VEROROUTE_ANDROID
 	void commonKeyPressEvent(QKeyEvent* event);		// So child dialogs can relay Ctrl and Shift to the main window
 	void commonKeyReleaseEvent(QKeyEvent* event);	// So child dialogs can relay Ctrl and Shift to the main window
 	void specialKeyPressEvent(QKeyEvent* event);	// So child dialogs can do Ctrl+Q etc
+#endif
 	void dragEnterEvent(QDragEnterEvent *e);
 	void dropEvent(QDropEvent *e);
+	bool eventFilter(QObject* object, QEvent* event);	// Used with installEventFilter to intercept the Android "back" button
 public slots:
 	void Startup();
 	// File menu items
@@ -159,9 +201,16 @@ public slots:
 	void ToggleDiagsMax();
 	void ToggleFill();
 	void ToggleSelectArea();
+	void TogglePaintGrid();
+	void ToggleEraseGrid();
+	void TogglePaintPins();
+	void ToggleErasePins();
+	void TogglePaintFlood();
+	void ResetMouseMode();
 	// Edit menu items
 	void Undo();
 	void Redo();
+	void SmartPanOn();
 	void Copy();
 	void Group();
 	void Ungroup();
@@ -218,30 +267,47 @@ public slots:
 		int iRow, iCol;
 		GetFirstRowCol(iRow, iCol);
 		m_board.AddTextBox(iRow, iCol);
-		UpdateHistory("Add text");
+		UpdateHistory("add text");
+		ShowTextDialog();
 		UpdateControls();
 		RepaintSkipRouting();
-		ShowTextDialog();
 	}
-	// Windows menu items
+	// Windows menu items + Other dialogs
+	void ToggleControlDialog();
+	void ToggleCompDialog();
+	void ToggleTemplatesDialog();
+	void ToggleRenderingDialog();
+	void ToggleInfoDialog();
+	void TogglePinDialog();
 	void ShowControlDialog();
-	void ShowTemplatesDialog();
-	void ShowRenderingDialog();
-	void ShowWireDialog();
-	void ShowHotkeysDialog();
-	void ShowInfoDialog();
 	void ShowCompDialog();
-	void ShowTextDialog();
-	void ShowBomDialog();
+	void HideCompDialog();
+	void ShowTemplatesDialog();
+	void HideTemplatesDialog();
+	void ShowRenderingDialog();
+	void HideRenderingDialog();
+	void ShowInfoDialog();
+	void HideInfoDialog();
 	void ShowPinDialog();
+	void HidePinDialog();
+	void ShowBomDialog();
+	void HideBomDialog();
+	void ShowWireDialog();
+	void HideWireDialog();
 	void ShowFindDialog();
+	void HideFindDialog();
+	void ShowTextDialog();
 	void ShowAbout();
 	void ShowSupport();
+	void ShowHotkeysDialog();
+	void ShowPadOffsetDialog();	// Triggered by clicking on a pin for 1000ms
+	void HidePadOffsetDialog(bool bForce = false);
 	// Layers menu items
 	void AddLayer();
 	void RemoveLayer();
 	void SwitchLayer();
 	void ToggleVias();
+	void ResetLayerPrefs();
 	// Help menu items
 	void LoadFirstTutorial();
 	void LoadPrevTutorial();
@@ -253,11 +319,20 @@ public slots:
 
 	// Helpers for child dialogs
 	void ShowDlg(QWidget* p);
+	void HideDlg(QWidget* p);
+	void HideAllDockedDlgs();
+	void HideAllNonDockedDlgs();
 
 	// View controls
-	void TrackSliderChanged(int i);
+	void TrackSliderChanged(int i);	// Actually a helper for the following 3 checkboxes
+	void CheckBoxMonoChanged(bool b);
+	void CheckBoxColorChanged(bool b);
+	void CheckBoxPcbChanged(bool b);
 	void SaturationSliderChanged(int i);
-	void CompSliderChanged(int i);
+	void CompSliderChanged(int i);	// Actually a helper for the following 3 checkboxes
+	void CheckBoxLineChanged(bool b);
+	void CheckBoxNameChanged(bool b);
+	void CheckBoxValueChanged(bool b);
 	void FillSliderChanged(int i);
 	void MarginChanged(int i);
 	void SetShowGrid(bool b);
@@ -285,8 +360,14 @@ public slots:
 	void CompTextR()		{ CompTextMove(0,  1); }
 	void CompTextT()		{ CompTextMove(-1, 0); }
 	void CompTextB()		{ CompTextMove( 1, 0); }
+	// Pin Shift
+	void PadCentre()		{ PadMove(0,  0); }
+	void PadMoveL()			{ PadMove(0, -1); }
+	void PadMoveR()			{ PadMove(0,  1); }
+	void PadMoveT()			{ PadMove(-1, 0); }
+	void PadMoveB()			{ PadMove( 1, 0); }
 	// Bad Nodes lists
-	void SetNodeId(QListWidgetItem* item);
+	void SetNodeId(QListWidgetItem* p);
 	void ListNodes(bool bRebuild = true);
 	// Routing controls
 	void EnableRouting(bool b);
@@ -309,6 +390,7 @@ public slots:
 	// Rendering options
 	void SetBrightness(int i);
 	void SetTrackWidth(int i);
+	void SetTagWidth(int i);
 	void SetHoleWidth(int i);
 	void SetPadWidth(int i);
 	void SetGapWidth(int i);
@@ -322,11 +404,14 @@ public slots:
 	void SetTargetRows(int i);
 	void SetTargetCols(int i);
 	void SetShowTarget(bool b);
+	void SetXthermals(bool b);
 	void SetShowCloseTracks(bool b);
 	void SetAntialiasOff(bool b);
 	void SetAntialiasOn(bool b);
 	// Find parts by name/value
+	void ClearFind();
 	void Find(bool bUseName, bool bExact, const QString& str);
+	size_t GetNumFound();
 	// Wire options
 	void SetWireShare(bool b);
 	void SetWireCross(bool b);
@@ -350,7 +435,7 @@ public slots:
 	TemplateManager& GetTemplateManager() { return m_templateMgr; }
 	// Info dialog
 	void SetInfoStr(const QString& str)	{ m_board.SetInfoStr(str.toStdString()); }
-	void OpenVrt(const QString& fileName, bool bMerge = false);	// Helper for opening a vrt using Open(), Merge(), dropEvent(), or the command line
+	void OpenVrt(const QString& fileName, bool bMerge, bool bCrashRecovery = false);	// Helper for opening a vrt using Open(), Merge(), dropEvent(), or the command line
 	// Component editor
 	void DefinerSetValueStr(const QString& str);
 	void DefinerSetPrefixStr(const QString& str);
@@ -367,6 +452,7 @@ public slots:
 	void DefinerToggleShapeFill(bool b);
 	void DefinerSetPinNumber(int i);
 	void DefinerIncPinNumber(bool b);
+	void DefinerSetSurface(const QString& str);
 	void DefinerSetShapeType(const QString& str);
 	void DefinerSetCX(double d);
 	void DefinerSetCY(double d);
@@ -386,19 +472,35 @@ public slots:
 	void DefinerChooseColor();
 	void DefinerRaise();
 	void DefinerLower();
+
+	// GUI update
+	void UpdateControls();
 private:
+	// GUI update
+	void UpdateRecentFiles(const QString* pFileName, bool bAdd);
+	void UpdateWindowTitle();
+	void UpdateRulerInfo();
+	void UpdateCompDialog();
+	void EnableCompDialogControls();
+	void UpdateBOM();
+	void UpdateTemplatesDialog();
+	void UpdateTextDialog(bool bFull = false);
+
 	void DestroyPixmapCache();
+#ifdef USE_PIXMAP_CACHE
 	void CreatePixmapCache(const GuiControl& guiCtrl, ColorManager& colorManager);
+	void PaintDiag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pCorner, bool bLT);
+#endif
 	void PaintViaGrey(const GuiControl& guiCtrl, QPainter& painter, const QPointF& pC);
 	void PaintPadGrey(const GuiControl& guiCtrl, QPainter& painter, QPen& pen, const QPointF& pC, const int& iPadWidthMIL = 0);
 	void PaintVia(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, const bool& bGap = false);	// Helper
 	void PaintPad(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, const int& iPadWidthMIL = 0, const int& iHoleWidth_MIL = 0, const bool& bGap = false);	// Helper
-	void PaintTag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const int& iPadWidthMIL, const int& iNbr, const int& iLyr);
-	void PaintDiag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pCorner, bool bLT);
-	void PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const QPointF& pCoffset, const int& iPerimeterCode, const bool bHavePad = false, const bool bGap = false);	// Helper
+	void PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const QPointF& pCoffset,
+				   const int& iPadWidthMIL, const int& iPerimeterCode, const int& iTagCode,
+				   const bool bHavePad, const bool bIsGnd, const bool bGap = false);	// Helper
 	void PaintBoard();
 	void PaintCompDefiner();
-	void HandleRouting();
+	void HandleRouting(const bool bSingleRoute = false);
 	void RepaintWithListNodes(bool bNow = false);
 	void RepaintWithRouting(bool bNow = false);
 	void RepaintSkipRouting(bool bNow = false);
@@ -413,7 +515,7 @@ private:
 	void SetCurrentCompId(const int& i)		{ m_board.SetCurrentCompId(i);	UpdateControls(); }
 	void SetCurrentTextId(const int& i)		{ m_board.SetCurrentTextId(i);	UpdateControls(); }
 	void SetCurrentPinId(const int& i)		{ m_board.SetCurrentPinId(i);	UpdateCompDialog(); }
-	void SetCurrentShapeId(const int& i)	{ m_board.SetCurrentShapeId(i);	UpdateCompDialog(); }
+	void SetCurrentShapeId(const int& i)	{ m_board.SetCurrentShapeId(i);	UpdateCompDialog(); UpdateControls(); }
 	TextRect& GetCurrentTextRect()			{ return m_board.GetTextMgr().GetTextRectById( GetCurrentTextId() ); }
 	// Helpers for slots
 	void ZoomHelper(int delta);
@@ -421,6 +523,8 @@ private:
 	{
 		if ( m_board.GetCompEdit() ) return;	// Do nothing in component editor mode
 
+		ResetMouseMode();
+
 		int iRow, iCol;
 		GetFirstRowCol(iRow, iCol);
 		const int compId = m_board.CreateComponent(iRow, iCol, eType);
@@ -430,12 +534,16 @@ private:
 		groupMgr.ResetUserGroup( compId );	// Reset the user group with the current comp (and its siblings)
 		SetCurrentCompId(compId);
 
-		UpdateHistory("Add part"); UpdateControls(); UpdateBOM(); RepaintSkipRouting();
+		UpdateHistory("add part"); UpdateControls(); UpdateBOM(); RepaintSkipRouting();
 	}
-	void AddFromTemplate(const Component& compTemp)
+	void AddFromTemplate(bool bGeneric, const Component& compTemp)
 	{
 		if ( m_board.GetCompEdit() ) return;	// Do nothing in component editor mode
 
+		if ( bGeneric ) return AddPart( compTemp.GetType() );
+
+		ResetMouseMode();
+
 		int iRow, iCol;
 		GetFirstRowCol(iRow, iCol);
 		const int compId = m_board.CreateComponent(iRow, iCol, compTemp.GetType(), &compTemp);
@@ -445,35 +553,29 @@ private:
 		groupMgr.ResetUserGroup( compId );	// Reset the user group with the current comp (and its siblings)
 		SetCurrentCompId(compId);
 
-		UpdateHistory("Add part"); UpdateControls(); UpdateBOM(); RepaintSkipRouting();
+		UpdateHistory("add part"); UpdateControls(); UpdateBOM(); RepaintSkipRouting();
 	}
 	void CompRotate(const bool& bCW);
 	void CompStretch(const bool& bGrow);
 	void CompStretchWidth(const bool& bGrow);
 	void CompTextMove(const int& deltaRow, const int& deltaCol);
-
-	// GUI update
-	void UpdateRecentFiles(const QString* pFileName, bool bAdd);
-	void UpdateWindowTitle();
-	void UpdateRulerInfo();
-	void UpdateControls();
-	void UpdateCompDialog();
-	void EnableCompDialogControls();
-	void UpdateBOM();
-	void UpdateTemplatesDialog();
-	void UpdateTextDialog(bool bFull = false);
+	void PadMove(const int& deltaRowMil, const int& deltaColMil);
+	void UpdatePadInfo();
 
 	// Helpers
 	void SetQuality(QPainter& p);
 	void ResetRuler();
 	bool CanModifyRuler() const;
+public:
 	bool CanZoomIn() const;
 	bool CanZoomOut() const;
+private:
 	bool GetIsModified() const;
 	bool GetMatchesVrtFile(const std::string& fileName) const;
 	void ResetHistory(const std::string& str);
-	void UpdateHistory(const std::string& str, const int compId = BAD_COMPID);
+	void UpdateHistory(const std::string& str, const int objId = -1);
 	void UpdateUndoRedoControls();
+	void SetMouseActionString(const std::string& str, const int objId = -1);
 
 	// Helper to auto-append suffix when writing a file
 	QString GetSaveFileName(const QString& caption, const QString& nameFilter, const QString& defaultSuffix);
@@ -483,11 +585,12 @@ private:
 	// Pens
 	QPen	m_rulerPen;
 	QPen	m_backgroundPen;
-	QPen	m_darkGreyPen;
+	QPen	m_greyPen;
 	QPen	m_blackPen;
 	QPen	m_whitePen;
 	QPen	m_redPen;
 	QPen	m_orangePen;
+	QPen	m_yellowPen;
 	QPen	m_lightBluePen;
 	QPen	m_varPen;
 	QPen	m_dotPen;
@@ -495,7 +598,7 @@ private:
 	QBrush	m_backgroundBrush;
 	QBrush	m_darkBrush;
 	QBrush	m_varBrush;
-private:
+
 	QNetworkAccessManager	m_networkMgr;	// For checking version against Sourceforge
 	QAction*				m_recentFileAction[MAX_RECENT_FILES];
 	QAction*				m_separator			= nullptr;	// At the end of the recent files list
@@ -503,19 +606,23 @@ private:
 	MyScrollArea*			m_scrollArea		= nullptr;	// The mainwindow contains a scrollable area ...
 	QLabel*					m_label				= nullptr;	// ... for a QLabel widget that ...
 	QPixmap					m_mainPixmap;					// ... contains a pixmap image of the whole board
+	QLabel*					m_labelInfo			= nullptr;	// For permanent status bar text
 	QLabel*					m_labelStatus		= nullptr;	// For permanent status bar text
-	QDockWidget*			m_dockControlDlg	= nullptr;
-	QDockWidget*			m_dockCompDlg		= nullptr;
-	ControlDialog*			m_controlDlg		= nullptr;
-	RenderingDialog*		m_renderingDlg		= nullptr;
+#ifdef VEROROUTE_DEBUG
+	QLabel*					m_labelDebug		= nullptr;	// For permanent status bar text
+#endif
+
+	ControlDialog*			m_controlDlg		= nullptr;	QDockWidget* m_dockControlDlg	= nullptr;
+	CompDialog*				m_compDlg			= nullptr;	QDockWidget* m_dockCompDlg		= nullptr;
+	TemplatesDialog*		m_templatesDlg		= nullptr;	QDockWidget* m_dockTemplatesDlg	= nullptr;
+	RenderingDialog*		m_renderingDlg		= nullptr;	QDockWidget* m_dockRenderingDlg	= nullptr;
+	InfoDialog*				m_infoDlg			= nullptr;	QDockWidget* m_dockInfoDlg		= nullptr;
+	PinDialog*				m_pinDlg			= nullptr;	QDockWidget* m_dockPinDlg		= nullptr;
 	WireDialog*				m_wireDlg			= nullptr;
 	HotkeysDialog*			m_hotkeysDlg		= nullptr;
-	InfoDialog*				m_infoDlg			= nullptr;
-	CompDialog*				m_compDlg			= nullptr;
 	TextDialog*				m_textDlg			= nullptr;
 	BomDialog*				m_bomDlg			= nullptr;
-	TemplatesDialog*		m_templatesDlg		= nullptr;
-	PinDialog*				m_pinDlg			= nullptr;
+	PadOffsetDialog*		m_padOffsetDlg		= nullptr;
 	FindDialog*				m_findDlg			= nullptr;
 
 	Board					m_board;			// *** The main object ***
@@ -525,11 +632,13 @@ private:
 	QString					m_fileName;			// The loaded/saved .vrt file
 	QString					m_pdfFileName;		// The saved PDF file
 	QString					m_gerberFileName;	// The saved Gerber file
+	int						m_mouseObjId = -1;	// For the undo/redo history
 	std::string				m_mouseActionString;// For the undo/redo history
 	std::string				m_localDataPathStr;	// The path to the "history" and "templates" folders
 	std::string				m_tutorialsPathStr;	// The path to the "tutorials" folder and "veroroute.png"
 
 	// Cached pixmaps containing pre-colored pads and blobs.
+#ifdef USE_PIXMAP_CACHE
 	QPixmap**	m_ppPixmapPad		= nullptr;	// A pad in the host element
 	QPixmap**	m_ppPixmapVia		= nullptr;	// A via in the host element
 	QPixmap**	m_ppPixmapDiag		= nullptr;	// For filling small diagonal gaps not covered by blob pixmaps
@@ -540,19 +649,18 @@ private:
 	int			m_radPixmapVia		= 0;		// ...
 	int			m_radPixmapDiag		= 0;		// ...
 	int			m_radPixmapBlob		= 0;		// ...
+#endif
 	QPoint		m_mousePos;
+	QPoint		m_clickedPos;
 	bool		m_bRepaint			= false;	// Flag to make paintEvent() do something useful
 	bool		m_bMouseClick		= false;	// Flag of click beginning
 	bool		m_bLeftClick		= false;
 	bool		m_bRightClick		= false;
 	bool		m_bCtrlKeyDown		= false;
 	bool		m_bShiftKeyDown		= false;
-	bool		m_bPaintPins		= false;	// true ==> allow paint the component pins (and the board)
-	bool		m_bPaintBoard		= false;	// true ==> allow paint the board only (not the component pins)
-	bool		m_bPaintFlood		= false;	// true ==> allow flood-fill all connected tracks & pins
-	bool		m_bPaintLyrPref		= false;	// true ==> allow paint layer preference to pins
-	bool		m_bDefiningRect		= false;	// true ==> user is defining a rectangle
-	bool		m_bResizingText		= false;	// true ==> user is resizing a text rectangle
+
+	MOUSE_MODE	m_eMouseMode		= MOUSE_MODE::SELECT;
+
 	bool		m_bWritePDF			= false;	// true ==> draw to PDF file instead of screen
 	bool		m_bWriteGerber		= false;	// true ==> draw to Gerber file instead of screen
 	bool		m_bTwoLayerGerber	= false;	// true ==> 2-layer Gerber output instead of 1-layer
@@ -560,6 +668,9 @@ private:
 	bool		m_bTemplatesDir		= false;	// true ==> have "templates" folder
 	bool		m_bRuler			= false;
 	bool		m_bModifyRulerA		= false;
+	bool		m_bUpdatingControls	= false;
+	bool		m_bReRoute			= false;
+	bool		m_bReListNodes		= false;
 	int			m_XGRIDOFFSET		= 0;		// So we can centre when writing to PDF
 	int			m_YGRIDOFFSET		= 0;		// So we can centre when writing to PDF
 	int			m_XCORRECTION		= 0;		// So we can fully render pads larger than 100 mil diameter
