@@ -19,11 +19,13 @@
 
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
+#include "padoffsetdialog.h"
 #include "GPainter.h"
 #include "SpanningTreeHelper.h"
 
 void MainWindow::DestroyPixmapCache()
 {
+#ifdef USE_PIXMAP_CACHE
 	if ( m_ppPixmapPad )	for (int i = 0; i <     NUM_PIXMAP_COLORS; i++)	delete m_ppPixmapPad[i];
 	if ( m_ppPixmapVia )	for (int i = 0; i <     NUM_PIXMAP_COLORS; i++)	delete m_ppPixmapVia[i];
 	if ( m_ppPixmapDiag )	for (int i = 0; i < 2 * NUM_PIXMAP_COLORS; i++)	delete m_ppPixmapDiag[i];
@@ -34,8 +36,10 @@ void MainWindow::DestroyPixmapCache()
 	delete[] m_ppPixmapBlob;	m_ppPixmapBlob	= nullptr;
 	delete   m_pPixmapDiagLT;	m_pPixmapDiagLT = nullptr;
 	delete   m_pPixmapDiagRT;	m_pPixmapDiagRT = nullptr;
+#endif
 }
 
+#ifdef USE_PIXMAP_CACHE
 void MainWindow::CreatePixmapCache(const GuiControl& guiCtrl, ColorManager& colorMgr)
 {
 	if ( m_ppPixmapPad ) return;	// Cache exists
@@ -94,6 +98,8 @@ void MainWindow::CreatePixmapCache(const GuiControl& guiCtrl, ColorManager& colo
 		}
 	}
 	const QColor backgroundColor = GetBackgroundColor();
+	const bool bHavePad(false), bIsGnd(false);
+	const int iTagCode(0), iPadWidthMIL(0);
 	for (int i = 0; i < 256; i++)	// Loop all possible perimeter codes
 	{
 		m_ppPixmapBlob[i] = new QPixmap(2*m_radPixmapBlob, 2*m_radPixmapBlob);
@@ -101,7 +107,7 @@ void MainWindow::CreatePixmapCache(const GuiControl& guiCtrl, ColorManager& colo
 
 		painter.begin(m_ppPixmapBlob[i]);
 		const QPointF pC(m_radPixmapBlob, m_radPixmapBlob);
-		PaintBlob(guiCtrl, painter, Qt::black, pC, pC, i);
+		PaintBlob(guiCtrl, painter, Qt::black, pC, pC, iPadWidthMIL, i, iTagCode, bHavePad, bIsGnd);
 		painter.end();
 
 		// Now turn the black blob area transparent, so we can overlay it over colored nodes.
@@ -112,6 +118,24 @@ void MainWindow::CreatePixmapCache(const GuiControl& guiCtrl, ColorManager& colo
 	releaseMouse();
 }
 
+void MainWindow::PaintDiag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pCorner, bool bLT)
+{
+	// Only use this method for pixmap based track rendering
+	const int&	H			= m_radPixmapDiag;
+	const int	trackWidth	= guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetTRACK_MIL() ) << 1;	// Track width in pixels
+
+	static QPen	pen(Qt::black, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
+	pen.setColor(color);
+	pen.setWidth(trackWidth);
+	painter.setPen(pen);
+	painter.setBrush(Qt::NoBrush);
+	if ( bLT )
+		painter.drawLine(pCorner + QPointF(-H,-H), pCorner + QPointF(H, H));
+	else // bRT
+		painter.drawLine(pCorner + QPointF(-H, H), pCorner + QPointF(H,-H));
+}
+#endif
+
 void MainWindow::PaintViaGrey(const GuiControl& guiCtrl, QPainter& painter, const QPointF& pC)
 {
 	assert(!m_bWriteGerber);
@@ -188,71 +212,12 @@ void MainWindow::PaintPad(const GuiControl& guiCtrl, QPainter& painter, const QC
 	}
 }
 
-void MainWindow::PaintTag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const int& iPadWidthMIL, const int& iNbr, const int& iLyr)
+void MainWindow::PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const QPointF& pCoffset,
+						   const int& iPadWidthMIL, const int& iPerimeterCode, const int& iTagCode,
+						   const bool bHavePad, const bool bIsGnd, const bool bGap)
 {
-	// Paints a short tag connecting a pad to the ground fill
-	const int w = ( iPadWidthMIL == 0 ) ? guiCtrl.GetPAD_MIL() : iPadWidthMIL;
-	const int X = guiCtrl.GetHalfPixelsFromMIL( w ) + guiCtrl.GetPixelsFromMIL( guiCtrl.GetGAP_MIL() );
-	const int D = static_cast<int>( X * sqrt(0.5) );
-	QPointF pD;	// The other end of the tag
-	switch( iNbr)
-	{
-		case NBR_L:		pD = pC + QPointF(-X,  0);	break;
-		case NBR_LT:	pD = pC + QPointF(-D, -D);	break;
-		case NBR_T:		pD = pC + QPointF( 0, -X);	break;
-		case NBR_RT:	pD = pC + QPointF( D, -D);	break;
-		case NBR_R:		pD = pC + QPointF( X,  0);	break;
-		case NBR_RB:	pD = pC + QPointF( D,  D);	break;
-		case NBR_B:		pD = pC + QPointF( 0,  X);	break;
-		case NBR_LB:	pD = pC + QPointF(-D,  D);	break;
-	}
-	if ( m_bWriteGerber )
-	{
-		QPolygonF polygon;
-		polygon.push_back(pC);
-		polygon.push_back(pD);
-		switch(iLyr)
-		{
-			case 0:	m_gWriter.GetStream(GFILE::GBL).AddTrack(polygon, GPEN::TRK); break;	// Bottom copper layer
-			case 1:	m_gWriter.GetStream(GFILE::GTL).AddTrack(polygon, GPEN::TRK); break;	// Top    copper layer
-		}
-	}
-	else
-	{
-		const int trackWidth = guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetTRACK_MIL() ) << 1;
-		static QPen	pen(Qt::black, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
-		pen.setColor(color);
-		pen.setWidth(trackWidth);
-		painter.setPen(pen);
-		painter.setBrush(Qt::NoBrush);
-		painter.drawLine(pC, pD);
-	}
-}
-
-void MainWindow::PaintDiag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pCorner, bool bLT)
-{
-	const int&	H			= m_radPixmapDiag;
-	const int	trackWidth	= guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetTRACK_MIL() ) << 1;	// Track width in pixels
-
-	static QPen	pen(Qt::black, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
-	pen.setColor(color);
-	pen.setWidth(trackWidth);
-	painter.setPen(pen);
-	painter.setBrush(Qt::NoBrush);
-	if ( bLT )
-		painter.drawLine(pCorner + QPointF(-H,-H), pCorner + QPointF(H, H));
-	else // bRT
-		painter.drawLine(pCorner + QPointF(-H, H), pCorner + QPointF(H,-H));
-}
-
-void MainWindow::PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const QPointF& pCoffset, const int& iPerimeterCode, const bool bHavePad, const bool bGap)
-{
-	const int	gapWidth	= bGap ? guiCtrl.GetPixelsFromMIL( guiCtrl.GetGAP_MIL() ) : 0;
-	const int	padWidth	= ( guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetPAD_MIL() )   + gapWidth ) << 1;	// Pad width in pixels
-	const int	trackWidth	= ( guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetTRACK_MIL() ) + gapWidth ) << 1;	// Track width in pixels
-
 	std::list<MyPolygonF> polygonList;
-	guiCtrl.CalcBlob(guiCtrl.GetGRIDPIXELS(), pC, pCoffset, iPerimeterCode, polygonList, bHavePad, bGap);	// Populate polygonList
+	guiCtrl.CalcBlob(guiCtrl.GetGRIDPIXELS(), pC, pCoffset, iPadWidthMIL, iPerimeterCode, iTagCode, polygonList, bHavePad, bIsGnd, bGap);	// Populate polygonList
 
 	if ( m_bWriteGerber )	// Write to Gerber
 	{
@@ -264,24 +229,30 @@ void MainWindow::PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const Q
 			{
 				const bool bPad	= polygon.m_ePadPen != GPEN::NONE;
 				const bool bTrk	= polygon.m_eTrkPen != GPEN::NONE;
-				if ( !bTrk && !bPad ) continue;
+				if ( !bTrk && !bPad && !polygon.m_bClosed ) continue;
 
 				const GPEN& ePen = bTrk ? polygon.m_eTrkPen : polygon.m_ePadPen;
 
 				if ( polygon.m_bClosed )
 				{
-					os.AddLoop(polygon, ePen);			// Closed polygon outline
-					if ( !bGap ) os.AddRegion(polygon);	// Only non-Gap polygon needs filling
+					if ( bPad || bTrk )
+						os.AddLoop(polygon, ePen);	// Closed polygon outline
+					if ( !bGap )
+						os.AddRegion(polygon);		// Only non-Gap polygon needs filling
 				}
 				else if ( bPad && bTrk )	// Fat tracks with diagonals
 					os.AddVariTrack(polygon, polygon.m_ePadPen, polygon.m_eTrkPen);
-				else
+				else if ( bPad || bTrk )
 					os.AddTrack(polygon, ePen);
 			}
 		}
 	}
 	else	// Draw to pixmap
 	{
+		const int	gapWidth	= bGap ? guiCtrl.GetPixelsFromMIL( guiCtrl.GetGAP_MIL() ) : 0;
+		const int	padWidth	= ( guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetPAD_MIL() )   + gapWidth ) << 1;	// Pad width in pixels
+		const int	trackWidth	= ( guiCtrl.GetHalfPixelsFromMIL( bIsGnd ? guiCtrl.GetTAG_MIL() : guiCtrl.GetTRACK_MIL() ) + gapWidth ) << 1;	// Track width in pixels
+
 		static QPen		pen(Qt::black, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 		static QBrush	brush(Qt::black,  Qt::SolidPattern);
 		pen.setColor(color);
@@ -292,9 +263,9 @@ void MainWindow::PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const Q
 		{
 			const bool bTrk	= polygon.m_eTrkPen != GPEN::NONE;
 			const bool bPad	= polygon.m_ePadPen != GPEN::NONE;
-			if ( !bTrk && !bPad ) continue;
+			if ( !bTrk && !bPad && !polygon.m_bClosed ) continue;
 
-			pen.setWidth(bTrk ? trackWidth : padWidth);
+			pen.setWidth(bTrk ? trackWidth : bPad ? padWidth : 0);
 			painter.setPen(pen);
 
 			if ( polygon.m_bClosed )
@@ -311,15 +282,18 @@ void MainWindow::PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const Q
 					painter.drawLine(*iterA, *iterB); ++iterA; ++iterB;
 				}
 			}
-			else if ( polygon.size() == 1 )
-				painter.drawPoint( polygon.first() );
-			else
+			else if ( bPad || bTrk )
 			{
-				auto iterA = polygon.begin();
-				auto iterB = iterA; iterB++;
-				while( iterB != polygon.end() )
+				if ( polygon.size() == 1 )
+					painter.drawPoint( polygon.first() );
+				else
 				{
-					painter.drawLine(*iterA, *iterB); ++iterA; ++iterB;
+					auto iterA = polygon.begin();
+					auto iterB = iterA; iterB++;
+					while( iterB != polygon.end() )
+					{
+						painter.drawLine(*iterA, *iterB); ++iterA; ++iterB;
+					}
 				}
 			}
 		}
@@ -341,6 +315,8 @@ void MainWindow::paintEvent(QPaintEvent*)
 	m_scrollArea->setWidgetResizable(false);
 	m_label->adjustSize();
 
+	m_scrollArea->CentreView();	// Centre the scrollbars (if needed) for component editor mode
+
 	m_bRepaint = false;
 }
 
@@ -349,10 +325,9 @@ void MainWindow::PaintCompDefiner()	// The paint method in "component editor mod
 	Board&			board		= m_board;
 	CompDefiner&	def			= board.GetCompDefiner();
 	const PinGrid&	grid		= def.GetGrid();
-
-	const int&		 W			= board.GetGRIDPIXELS();	// Square width in pixels
-	const int		 C			= W >> 1;					// Half square width in pixels
-	const double	 dTextScale	= W / 24.0;					// For scaling text when zooming
+	const int&		W			= board.GetGRIDPIXELS();	// Square width in pixels
+	const int		C			= W >> 1;					// Half square width in pixels
+	const double	dTextScale	= W / 24.0;					// For scaling text when zooming
 
 	// Shift comp to near grid centre
 	const int ROWS(def.GetScreenRows()), COLS(def.GetScreenCols());
@@ -468,34 +443,53 @@ void MainWindow::PaintCompDefiner()	// The paint method in "component editor mod
 
 	m_varBrush.setColor(QColor(192,192,255,128));	// Light blue
 
-	int iPinId(0);
-	for (int j = 0, jMax = grid.GetRows(); j < jMax; j++)
-	for (int i = 0, iMax = grid.GetCols(); i < iMax; i++, iPinId++)
+	for (int iLoop = 0; iLoop < 2; iLoop++)
 	{
-		auto p = grid.Get(0,j,i);	// Always layer 0 for component editor
-		GetXY(board, def.GetGridRowMin() + j, def.GetGridColMin() + i, X, Y );
-
-		// Write pin labels
-		painter.save();
-		painter.translate(X, Y);
-
-		painter.setPen(Qt::NoPen);
-		painter.setBrush(p->GetSurface() == SURFACE_FREE ? Qt::NoBrush :
-						 p->GetSurface() == SURFACE_HOLE ? m_darkBrush : m_varBrush);	//TODO Handle SURFACE_GAP, SURFACE PLUG in future
-		painter.drawRect(-C,-C,W,W);
-		if ( iPinId == def.GetCurrentPinId() )
+		int iPinId(0);
+		for (int j = 0, jMax = grid.GetRows(); j < jMax; j++)
+		for (int i = 0, iMax = grid.GetCols(); i < iMax; i++, iPinId++)
 		{
-			painter.setPen(m_redPen);
-			painter.drawEllipse(-C,-C,W,W);
+			auto p = grid.Get(0,j,i);	// Always layer 0 for component editor
+			GetXY(board, def.GetGridRowMin() + j, def.GetGridColMin() + i, X, Y );
+
+			painter.save();
+			painter.translate(X, Y);
+
+			if ( iLoop == 0 )	// Shade according to GetSurface()
+			{
+				painter.setPen(Qt::NoPen);
+				painter.setBrush(p->GetSurface() == SURFACE_FREE ? Qt::NoBrush :
+								 p->GetSurface() == SURFACE_HOLE ? m_darkBrush : m_varBrush);	//TODO Handle SURFACE_GAP, SURFACE PLUG in future
+				painter.drawRect(-C,-C,W,W);
+			}
+			if ( iLoop == 1 )	// Draw pins/labels
+			{
+				if ( iPinId == def.GetCurrentPinId() )
+				{
+					painter.setPen(m_redPen);
+					painter.drawEllipse(-C,-C,W,W);
+				}
+				if ( p->GetIsPin() )
+				{
+					painter.setPen(m_blackPen);
+					painter.rotate(270);
+
+					const size_t pinIndex = p->GetPinIndex();
+					std::string strPinLabel = ( pinIndex < def.GetNumPins() ) ? def.GetPinLabel(pinIndex) : CompTypes::GetDefaultPinLabel(pinIndex);
+					int iFlag = ( pinIndex < def.GetNumPins() ) ? def.GetPinAlign(pinIndex) : Qt::AlignCenter;
+					if ( iFlag == Qt::AlignLeft || iFlag == Qt::AlignRight )
+					{
+						const bool bLeft = ( iFlag == Qt::AlignLeft );
+						painter.translate(bLeft ? -C/2 : C/2, 0);
+					}
+					iFlag |= ( Qt::TextDontClip | Qt::AlignVCenter );
+
+					painter.scale(dTextScale, dTextScale);
+					painter.drawText(0,0,0,0, iFlag, strPinLabel.c_str());
+				}
+			}
+			painter.restore();
 		}
-		if ( p->GetIsPin() )
-		{
-			painter.setPen(m_blackPen);
-			painter.rotate(270);
-			painter.scale(dTextScale, dTextScale);
-			painter.drawText(0,0,0,0, Qt::TextDontClip | Qt::AlignCenter, CompTypes::GetDefaultPinLabel(p->GetPinIndex()).c_str());
-		}
-		painter.restore();
 	}
 	painter.end();
 }
@@ -515,11 +509,15 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 	const bool		 bPCB			= trackMode == TRACKMODE::PCB;
 	const bool		 bMonoPCB		= bMono || bPCB;
 	const bool		 bGroundFill	= !bVero && bMonoPCB && board.GetGroundFill();
+	const bool		 bForceXthermal	= board.GetXthermals();
+#ifdef USE_PIXMAP_CACHE
 	const bool		 bPixmapCache	= !bVero && ( bMono || bColor ) && !bGroundFill && !m_bWritePDF;	// true ==> Faster rendering (Mono/Color modes)
+#else
+	const bool		 bPixmapCache	= false;
+#endif
 	const bool		 bDirect		= !bVero && !bPixmapCache && !bGroundFill;	// true ==> Draw track "blobs" and pads directly (PDF/Gerber)
-	const bool		 bExtraTags		= false;									// true ==> Add extra thermal relief tags
 	const int&		 layer			= board.GetCurrentLayer();
-	const int&		 groundNodeId	= ( layer == 0 ) ? board.GetGroundNodeId0() :  board.GetGroundNodeId1();
+	const int&		 groundNodeId	= board.GetGroundNodeId(layer);
 	const bool		 bWiresAsTracks	= m_bWriteGerber && m_bTwoLayerGerber && board.GetLyrs() == 1;	// true ==> Convert wires to tracks on the top layer
 	const int&		 W				= board.GetGRIDPIXELS();		// Square width in pixels
 	const int		 C				= W >> 1;						// Half square width in pixels
@@ -541,8 +539,10 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 
 	colorMgr.SetSaturation( board.GetSaturation() );			// Must do this BEFORE making pixmaps
 	colorMgr.SetFillSaturation( board.GetFillSaturation() );	// Must do this BEFORE making pixmaps
+#ifdef USE_PIXMAP_CACHE
 	if ( bPixmapCache )
 		CreatePixmapCache(board, colorMgr);	// Builds pixmaps if the cache is empty
+#endif
 
 	board.CalculateColors();	// Work out best way to color things
 
@@ -704,6 +704,10 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				const bool		bVia			= pC->GetIsVia()  ||  bWireAsVia;
 				const bool		bPad			= !bWireAsVia && pC->GetHasPin();
 				assert( !(bVia && bPad) );	// Can't be both a via and a regular pad
+				const bool		bIsGnd			= bGroundFill && nodeId == groundNodeId;
+				const int		iTagCode		= ( bPad && bIsGnd && nodeId != BAD_NODEID ) ? board.GetTagCode(pC, iPerimeterCode) : 0;
+
+				if ( bForceXthermal && !bPad && bIsGnd ) continue;	// Don't render ground tracks if forcing X shaped thermal reliefs
 
 				bool	bCustomSize(false);
 				int		iPadWidthMIL(0), iHoleWidthMIL(0);	// 0 ==> Not a custom size value
@@ -720,15 +724,12 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					if ( !bVero )
 					{
 						comp.GetCompPinOffsets(pinIndex, padOffsetX, padOffsetY);	// Get offsets in mil
-						padOffsetX = (padOffsetX * W) / 100;	 // Convert from mil to pixels
-						padOffsetY = (padOffsetY * W) / 100;	 // Convert from mil to pixels
+						padOffsetX = (padOffsetX * W) / 100;	// Convert from mil to pixels
+						padOffsetY = (padOffsetY * W) / 100;	// Convert from mil to pixels
 					}
 
 					if ( board.GetLyrs() == 2 )
-					{
-						if ( nodeId != BAD_NODEID )
-							layerPref	= comp.GetLayerPref(pinIndex);
-					}
+						layerPref = comp.GetLayerPref(pinIndex);
 
 					bCustomSize = comp.GetCustomPads();
 					if ( bCustomSize )
@@ -738,7 +739,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					}
 				}
 				const bool bPadOffset	= ( padOffsetX != 0 || padOffsetY != 0 );
-				const bool bBlob		= !bPadOffset || iPerimeterCode != 0;
+				const bool bBlob		= !bPadOffset || iPerimeterCode != 0 || ( bForceXthermal && bIsGnd );
 
 				QPen& greyPen = ( layerPref == LAYER_X ) ? penGry :
 								( layerPref == LAYER_T ) ? penTop : penBot;
@@ -756,7 +757,6 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 												&&	iEffColorId != MY_LYR_BOT	&&	iEffColorId != MY_LYR_TOP;
 				const QColor color				= bAllowCustomColor ? colorMgr.GetColorFromNodeId(nodeId)
 																	: colorMgr.GetPixmapColor(iEffColorId);
-				const bool	 bCustomColor		= bAllowCustomColor && colorMgr.GetIsFixed(nodeId);
 
 				GetLRTB(board, 100, j, i, L, R, T, B);	// 100% size square
 				const int X((L+R)/2), Y((T+B)/2);
@@ -813,7 +813,9 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				}
 				if ( bPixmapCache )	// Draw track "blobs" and pads using pre-calculated pixmaps for speed
 				{
+#ifdef USE_PIXMAP_CACHE
 					assert( !bPCB );
+					const bool bCustomColor = bAllowCustomColor && colorMgr.GetIsFixed(nodeId);
 					if ( iLoop == 0 )
 					{
 						painter.setPen(Qt::NoPen);
@@ -874,40 +876,25 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					}
 					else if (  bCustomSize || bPadOffset )	// Custom/offset stuff ...
 					{
-						if ( bBlob ) PaintBlob(board, painter, color, pCentre, pCentreOff, iPerimeterCode, bPad);
+						if ( bBlob ) PaintBlob(board, painter, color, pCentre, pCentreOff, iPadWidthMIL, iPerimeterCode, iTagCode, bPad, false);
 						if ( bPad )  PaintPad(board, painter, color, pCentreOff, iPadWidthMIL, iHoleWidthMIL);
 					}
+#endif	// USE_PIXMAP_CACHE
 				}
 				if ( bGroundFill )	// Draw track "blobs" and pads directly (PDF/Gerber)
 				{
 					if ( iLoop == 0 )
 					{
-						if ( nodeId != groundNodeId && bBlob )	// Only the non-ground tracks have a "white" surround
-							PaintBlob(board, painter, backgroundColor, pCentre, pCentreOff, iPerimeterCode, bPad, true);		// Draw fat "white" track blob
-						if ( bVia ) PaintVia(board, painter, backgroundColor, pCentre, true);									// Draw fat "white" via
-						if ( bPad ) PaintPad(board, painter, backgroundColor, pCentreOff, iPadWidthMIL, iHoleWidthMIL, true);	// Draw fat "white" pad
+						if ( !bIsGnd && bBlob )	// Only the non-ground tracks have a "white" surround
+							PaintBlob(board, painter, backgroundColor, pCentre, pCentreOff, iPadWidthMIL, iPerimeterCode, iTagCode, bPad, bIsGnd, true);	// Draw fat "white" track blob
+						if ( bVia ) PaintVia(board, painter, backgroundColor, pCentre, true);										// Draw fat "white" via
+						if ( bPad ) PaintPad(board, painter, backgroundColor, pCentreOff, iPadWidthMIL, iHoleWidthMIL, true);		// Draw fat "white" pad
 					}
 					else if ( iLoop == 1 )	// Draw track "blobs" and pads directly
 					{
-						if ( bBlob ) PaintBlob(board, painter, color, pCentre, pCentreOff, iPerimeterCode, bPad);			// Draw track blob
-						if ( bVia )  PaintVia(board, painter, color, pCentre);												// Draw via same color as track
-						if ( bPad )  PaintPad(board, painter, color, pCentreOff, iPadWidthMIL, iHoleWidthMIL);				// Draw pad same color as track
-						if ( bExtraTags && bPad && nodeId == groundNodeId && iPerimeterCode > 0 && nodeId != BAD_NODEID )	// Draw extra thermal relief tags
-						{
-							int iCode(iPerimeterCode);		// Take a copy of the perimeter code
-							const bool bDiagsOK( board.GetDiagsMode() != DIAGSMODE::OFF );	// If bExtraTags gets set true then move this to start of method
-							for (int iDiag = 0, iDiagMax = ( bDiagsOK ) ? 2 : 1; iDiag < iDiagMax; iDiag++)	// First pass ==> Non-diagonal nbrs.  Second pass diagonal nbrs
-							for (int iNbr = iDiag; iNbr < 8; iNbr += 2)	// Even/Odd iNbr ==> Non-diagonal/Diagonal
-							{
-								const Element* pNbr = pC->GetNbr(iNbr);
-								if ( pNbr->GetNodeId() != BAD_NODEID ) continue;	// Skip if direction is not empty
-								if ( pC->IsBlocked(iNbr, nodeId) ) continue;		// Skip is direction is blocked
-								if ( ReadCodeBit((iNbr+1)%8 , iCode) ) continue;	// Skip if adjacent CW  direction already has connection
-								if ( ReadCodeBit((iNbr+7)%8, iCode) ) continue;		// Skip if adjacent CCW direction already has connection
-								SetCodeBit(iNbr, iCode);	// Update the copy of the perimeter code
-								PaintTag(board, painter, color, pCentre, iPadWidthMIL, iNbr, layer);	// Draw tag to ground fill
-							}
-						}
+						if ( bBlob ) PaintBlob(board, painter, color, pCentre, pCentreOff, iPadWidthMIL, iPerimeterCode, iTagCode, bPad, bIsGnd);	// Draw track blob
+						if ( bVia )  PaintVia(board, painter, color, pCentre);									// Draw via same color as track
+						if ( bPad )  PaintPad(board, painter, color, pCentreOff, iPadWidthMIL, iHoleWidthMIL);	// Draw pad same color as track
 					}
 					else if ( bDrawGrey )
 					{
@@ -919,9 +906,9 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				{
 					if ( iLoop == 0 )
 					{
-						if ( bBlob ) PaintBlob(board, painter, color, pCentre, pCentreOff, iPerimeterCode, bPad);	// Draw track blob
-						if ( bVia )  PaintVia(board, painter, color, pCentre);										// Draw via same color as track
-						if ( bPad )  PaintPad(board, painter, color, pCentreOff, iPadWidthMIL, iHoleWidthMIL);		// Draw pad same color as track
+						if ( bBlob ) PaintBlob(board, painter, color, pCentre, pCentreOff, iPadWidthMIL, iPerimeterCode, iTagCode, bPad, false);	// Draw track blob
+						if ( bVia )  PaintVia(board, painter, color, pCentre);									// Draw via same color as track
+						if ( bPad )  PaintPad(board, painter, color, pCentreOff, iPadWidthMIL, iHoleWidthMIL);	// Draw pad same color as track
 					}
 					else if ( bDrawGrey )
 					{
@@ -986,39 +973,31 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 	}
 
 	// Draw hatched lines ========================================================================
-	if ( !m_bWriteGerber && trackMode != TRACKMODE::OFF )
+	if ( !m_bWriteGerber && trackMode != TRACKMODE::OFF && board.GetRoutingEnabled() )
 	{
-		if ( board.GetRoutingEnabled() || GetCurrentNodeId() != BAD_NODEID )
+		painter.save();
+		m_redPen.setWidth(W / 8);
+		m_backgroundPen.setWidth(0);
+		painter.setBrush(Qt::NoBrush);
+		for (int j = minRow; j <= maxRow; j++)
+		for (int i = minCol; i <= maxCol; i++)
 		{
-			painter.save();
-			m_redPen.setWidth(W / 8);
-			m_backgroundPen.setWidth(0);
-			painter.setBrush(Qt::NoBrush);
-			for (int j = minRow; j <= maxRow; j++)
-			for (int i = minCol; i <= maxCol; i++)
-			{
-				const Element* pC = board.Get(layer, j, i);
-				if ( pC->GetNodeId() == BAD_NODEID ) continue;
-				GetLRTB(board, 100, j, i, L, R, T, B);	// 100% size square
+			const Element* pC = board.Get(layer, j, i);
+			if ( pC->GetNodeId() == BAD_NODEID ) continue;
+			GetLRTB(board, 100, j, i, L, R, T, B);	// 100% size square
 
-				if ( pC->ReadFlagBits(AUTOSET) && !pC->ReadFlagBits(USERSET) )
-				{
-					painter.setPen(m_backgroundPen);
-					painter.drawLine(L, T, R, B);		// Draw "\" (hatched) line
-					painter.drawLine(L+C, T, R, B-C);	// Draw "\" (hatched) line
-					painter.drawLine(L, T+C, R-C, B);	// Draw "\" (hatched) line
-					painter.drawLine(L, B, R, T);		// Draw "/" (hatched) line
-					painter.drawLine(L+C, B, R, T+C);	// Draw "/" (hatched) line
-					painter.drawLine(L, B-C, R-C, T);	// Draw "/" (hatched) line
-				}
-				if ( pC->GetNodeId() == GetCurrentNodeId() && pC->GetMH() == BAD_MH )
-				{
-					painter.setPen(m_redPen);
-					painter.drawLine(L, B, R, T);		// Draw "/" (hatched) line
-				}
+			if ( pC->ReadFlagBits(AUTOSET) && !pC->ReadFlagBits(USERSET) )
+			{
+				painter.setPen(m_backgroundPen);
+				painter.drawLine(L, T, R, B);		// Draw "\" (hatched) line
+				painter.drawLine(L+C, T, R, B-C);	// Draw "\" (hatched) line
+				painter.drawLine(L, T+C, R-C, B);	// Draw "\" (hatched) line
+				painter.drawLine(L, B, R, T);		// Draw "/" (hatched) line
+				painter.drawLine(L+C, B, R, T+C);	// Draw "/" (hatched) line
+				painter.drawLine(L, B-C, R-C, T);	// Draw "/" (hatched) line
 			}
-			painter.restore();
 		}
+		painter.restore();
 	}
 
 	// Draw solder ===============================================================================
@@ -1108,8 +1087,8 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					penPlaced.setWidth(0);		// For pin labels
 					m_orangePen.setWidth(0);	// For pin labels and pins
 					m_redPen.setWidth(0);		// For pin labels and pins
-					m_darkGreyPen.setWidth(0);	// For pins
-					painter.setPen(bFound ? m_orangePen : bPlaced ? m_darkGreyPen : m_redPen);
+					m_greyPen.setWidth(0);		// For pins
+					painter.setPen(bFound ? m_orangePen : bPlaced ? m_greyPen : m_redPen);
 				}
 				painter.setBrush(Qt::NoBrush);
 
@@ -1357,6 +1336,52 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 		}
 	}
 
+	// Draw air wires ============================================================================
+	if ( !m_bWriteGerber && trackMode != TRACKMODE::OFF && GetCurrentNodeId() != BAD_NODEID )
+	{
+		painter.save();
+		m_redPen.setWidth( static_cast<int>(0.175 * W) );
+		m_redPen.setCapStyle(Qt::SquareCap);
+		m_redPen.setStyle(Qt::DotLine);
+		painter.setPen(m_redPen);
+		painter.setBrush(Qt::NoBrush);
+
+		int padOffsetX, padOffsetY;	// For handling offset pads
+
+		std::list<SpanningTreeHelper::AIRWIRE_POINT> spanTreePoints;
+
+		for (int jj = minRow; jj <= maxRow; jj++)
+		for (int ii = minCol; ii <= maxCol; ii++)
+		{
+			Element*	pD		= board.Get(0, jj, ii);	// Layer 0 only
+			const bool	bPin	= pD->GetHasPin() && !pD->GetHasWire();
+			if ( !bPin ) continue;
+			if ( pD->GetNodeId() != GetCurrentNodeId() ) continue;
+
+			GetXY(board, jj, ii, X, Y);
+
+			if ( !bVero )
+			{
+				const Component& comp = compMgr.GetComponentById( pD->GetCompId() );
+				comp.GetCompPinOffsets(pD->GetPinIndex(), padOffsetX, padOffsetY);	// Get offsets in mil
+				X += (padOffsetX * W) / 100;	// Convert from mil to pixels
+				Y += (padOffsetY * W) / 100;	// Convert from mil to pixels
+			}
+
+			spanTreePoints.push_back( SpanningTreeHelper::AIRWIRE_POINT(QPointF(X, Y), pD->GetRouteId()) );
+		}
+
+		std::list< SpanningTreeHelper::AIRWIRE_LINE > spanTreeLines;
+		SpanningTreeHelper::BuildAirWires(spanTreePoints, spanTreeLines);	// Build air-wires
+
+		for (const auto& o : spanTreeLines)
+			painter.drawLine(o.first.first, o.second.first);
+
+		m_redPen.setCapStyle(Qt::RoundCap);
+		m_redPen.setStyle(Qt::SolidLine);
+		painter.restore();
+	}
+
 	// Draw flying wires =========================================================================
 	if ( !m_bWriteGerber && !bMonoPCB && compMode != COMPSMODE::OFF && board.GetShowFlyWires() )
 	{
@@ -1365,6 +1390,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 		QPen  blackPen = m_blackPen;	// Used for lines in the component pixmap
 		blackPen.setWidth(2);
 		m_varPen.setWidth(static_cast<int>(0.175 * W));
+		m_varPen.setCapStyle(Qt::SquareCap);
 		m_varPen.setStyle(Qt::DotLine);
 		painter.setBrush(Qt::NoBrush);
 		const int		dH = static_cast<int>(0.1*W);	// Param for wire rounded rect
@@ -1396,8 +1422,8 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 			{
 				const Component& comp = compMgr.GetComponentById( pC->GetCompId() );
 				comp.GetCompPinOffsets(pC->GetPinIndex(), padOffsetX, padOffsetY);	// Get offsets in mil
-				X += (padOffsetX * W) / 100;	 // Convert from mil to pixels
-				Y += (padOffsetY * W) / 100;	 // Convert from mil to pixels
+				X += (padOffsetX * W) / 100;	// Convert from mil to pixels
+				Y += (padOffsetY * W) / 100;	// Convert from mil to pixels
 			}
 
 			spanTreePoints.push_back( QPointF(X, Y) );
@@ -1414,8 +1440,8 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				{
 					const Component& comp = compMgr.GetComponentById( pD->GetCompId() );
 					comp.GetCompPinOffsets(pD->GetPinIndex(), padOffsetX, padOffsetY);	// Get offsets in mil
-					X += (padOffsetX * W) / 100;	 // Convert from mil to pixels
-					Y += (padOffsetY * W) / 100;	 // Convert from mil to pixels
+					X += (padOffsetX * W) / 100;	// Convert from mil to pixels
+					Y += (padOffsetY * W) / 100;	// Convert from mil to pixels
 				}
 
 				spanTreePoints.push_back( QPointF(X, Y) );
@@ -1436,6 +1462,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				painter.restore();
 			}
 		}
+		m_varPen.setCapStyle(Qt::RoundCap);
 		painter.restore();
 	}
 
@@ -1625,6 +1652,31 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 		}
 	}
 
+	// Draw a cross on the pad being used for pad offset =========================================
+	if ( !m_bWriteGerber && !bVero && m_padOffsetDlg->isVisible() )
+	{
+		const Element* pC = m_board.Get(layer, m_gridRow, m_gridCol);
+		if ( pC->GetHasPin() && !pC->GetHasWire() )
+		{
+			GetXY(board, m_gridRow, m_gridCol, X, Y);
+
+			int padOffsetX, padOffsetY;	// For handling offset pads
+			const Component& comp = compMgr.GetComponentById( pC->GetCompId() );
+			comp.GetCompPinOffsets(pC->GetPinIndex(), padOffsetX, padOffsetY);	// Get offsets in mil
+
+			X += (padOffsetX * W) / 100;	// Convert from mil to pixels
+			Y += (padOffsetY * W) / 100;	// Convert from mil to pixels
+
+			painter.save();
+			painter.setBrush(Qt::NoBrush);
+			m_yellowPen.setWidth(2);
+			painter.setPen(m_yellowPen);
+			painter.drawLine(X-C, Y, X+C, Y);
+			painter.drawLine(X, Y-C, X, Y+C);
+			painter.restore();
+		}
+	}
+
 	// Draw places of min track separation =======================================================
 	if ( !m_bWriteGerber && !bVero && trackMode != TRACKMODE::OFF && m_board.GetShowCloseTracks() )
 	{
