@@ -30,7 +30,7 @@ class TextDialog : public QDialog
 	Q_OBJECT
 
 public:
-	explicit TextDialog(MainWindow* parent = 0);
+	explicit TextDialog(MainWindow* parent = nullptr);
 	~TextDialog();
 
 	void Clear();
@@ -45,5 +45,3 @@ private:
 	Ui_TextDialog*	ui;
 	MainWindow*		m_pMainWindow;
 };
-
-
